@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -157,6 +157,30 @@ let is_varname str =
 let failwithf fmt = Printf.ksprintf failwith fmt
 
 
+let rec file_location ?pos1 ?pos2 ?lexbuf () =
+    match pos1, pos2, lexbuf with
+    | Some p, None, _ | None, Some p, _ ->
+      file_location ~pos1:p ~pos2:p ?lexbuf ()
+    | Some p1, Some p2, _ ->
+      let open Lexing in
+      let fn, lineno = p1.pos_fname, p1.pos_lnum in
+      let c1 = p1.pos_cnum - p1.pos_bol in
+      let c2 = c1 + (p2.pos_cnum - p1.pos_cnum) in
+      Printf.sprintf (f_ "file %S, line %d, characters %d-%d")  fn lineno c1 c2
+    | _, _, Some lexbuf ->
+      file_location
+        ~pos1:(Lexing.lexeme_start_p lexbuf)
+        ~pos2:(Lexing.lexeme_end_p lexbuf)
+        ()
+    | None, None, None ->
+      s_ "<position undefined>"
+
+
+let failwithpf ?pos1 ?pos2 ?lexbuf fmt =
+  let loc = file_location ?pos1 ?pos2 ?lexbuf () in
+  Printf.ksprintf (fun s -> failwith (Printf.sprintf "%s: %s" loc s)) fmt
+
+
 (* END EXPORT *)
 
 
@@ -183,8 +207,6 @@ struct
     in
       Buffer.contents buf
 
-  let is_space c = c = ' ' || c = '\t' || c = '\n' || c = '\r'
-
   (* [escape s] escapes [s] in such a way that [unescape] recovers the
      original string. *)
   let escape s =
@@ -202,7 +224,7 @@ struct
            true
          | c ->
            Buffer.add_char buf c;
-           need_to_quote || is_space c || c = '\'')
+           need_to_quote || OASISString.is_whitespace c || c = '\'')
         (s = "") (* empty strings must be quoted *)
         s
     in
@@ -279,7 +301,7 @@ struct
     let rec skip_blank strm =
       match Stream.peek strm with
         | Some c ->
-            if is_space c then
+            if OASISString.is_whitespace c then
               begin
                 Stream.junk strm;
                 skip_blank strm
@@ -369,16 +391,13 @@ struct
               get_doubly_quoted_string strm
 
           | c ->
-              if is_space c then
-                begin
-                  (* We reach the end of an arg *)
-                  rargs := buf_flush () :: !rargs;
-                  skip_blank strm
-                end
-              else
-                begin
-                  buf_add c
-                end
+            if OASISString.is_whitespace c then begin
+              (* We reach the end of an arg *)
+              rargs := buf_flush () :: !rargs;
+              skip_blank strm
+            end else begin
+              buf_add c
+            end
       done;
       let last = buf_flush () in
       if last <> "" then rargs := last :: !rargs
