@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -44,7 +44,7 @@ let update_fail _ _ =
 let blackbox =
   {
     parse  =
-      (fun ~ctxt s ->
+      (fun ~ctxt:_ s ->
          failwithf
            (f_ "Blackbox type cannot be set to the value '%s'")
            s);
@@ -61,7 +61,7 @@ struct
 end
 
 
-let lexer ?(fail=(fun ~ctxt _ _ -> ())) lxr nm =
+let lexer ?(fail=(fun ~ctxt:_ _ _ -> ())) lxr nm =
   {
     parse =
       (fun ~ctxt str ->
@@ -113,7 +113,7 @@ let copyright =
 
 let string =
   {
-    parse =  (fun ~ctxt s -> s);
+    parse =  (fun ~ctxt:_ s -> s);
     update = (fun s1 s2 -> s1^" "^s2);
     print =  (fun s -> s);
   }
@@ -122,7 +122,7 @@ let string =
 let string_not_empty =
   {
     parse =
-      (fun ~ctxt str ->
+      (fun ~ctxt:_ str ->
          if str <> "" then
            str
          else
@@ -140,6 +140,18 @@ let file_glob =
   {string_not_empty with update = update_fail}
 
 
+let file_pattern =
+  {
+    parse =
+      (fun ~ctxt:_ str ->
+        match OASISString.nsplit str '%' with
+        | [pfx; sfx] -> (pfx, sfx)
+        | _ -> failwith (s_ "Expecting a file pattern, containing one %."));
+    update = update_fail;
+    print = (fun (pfx, sfx) -> pfx ^ "%" ^ sfx);
+  }
+
+
 let directory =
   {string_not_empty with update = update_fail}
 
@@ -208,7 +220,7 @@ let newline_separated value =
 let space_separated =
   {
     parse =
-      (fun ~ctxt s ->
+      (fun ~ctxt:_ s ->
          List.filter
            (fun s -> s <> "")
            (OASISString.nsplit s ' '));
@@ -258,15 +270,11 @@ let modules =
   let base_value =
     lexer
       StdLexer.modul
-      ~fail:(fun ~ctxt str e ->
-               match e with
-                 | Failure "lexing: empty token" ->
-                     if OASISString.capitalize_ascii str <> str then
-                       failwithf
-                         (f_ "Module name '%s', must be capitalized ('%s').")
-                         str (OASISString.capitalize_ascii str)
-                 | _ ->
-                     ())
+      ~fail:(fun ~ctxt:_ str _ ->
+          if OASISString.capitalize_ascii str <> str then
+            failwithf
+              (f_ "Module name '%s', must be capitalized ('%s').")
+              str (OASISString.capitalize_ascii str))
       (fun () -> s_ "module")
   in
     comma_separated
@@ -302,7 +310,7 @@ let categories =
 let choices nm lst =
   {
     parse =
-      (fun ~ctxt str ->
+      (fun ~ctxt:_ str ->
          try
            List.assoc
              (OASISString.lowercase_ascii str)
@@ -340,7 +348,7 @@ let boolean =
 let findlib_name =
   {
     parse =
-      (fun ~ctxt s ->
+      (fun ~ctxt:_ s ->
          if s = "" then
            failwith (s_ "Empty string is not a valid findlib package")
          else if String.contains s '"' || String.contains s '.' then
@@ -470,7 +478,7 @@ let command_line =
 
     {
       parse =
-        (fun ~ctxt s ->
+        (fun ~ctxt:_ s ->
            match split_expandable s with
              | cmd :: args ->
                  cmd, args
@@ -486,7 +494,7 @@ let command_line =
 
 
 let command_line_options =
-  { parse = (fun ~ctxt s -> POSIXShell.split s);
+  { parse = (fun ~ctxt:_ s -> POSIXShell.split s);
     update = List.append;
     print = (fun lst -> String.concat " " (List.map POSIXShell.escape lst));
   }
