@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -22,7 +22,7 @@
 
 
 (** Functions common to OCamlbuild build and doc plugin
-  *)
+*)
 
 
 open OASISGettext
@@ -31,9 +31,7 @@ open BaseStandardVar
 open OASISTypes
 
 
-TYPE_CONV_PATH "OCamlbuildCommon"
-
-type extra_args = string list with odn
+type extra_args = string list
 
 
 let ocamlbuild_clean_ev = "ocamlbuild-clean"
@@ -55,6 +53,14 @@ let fix_args args extra_argv =
           "-classic-display";
           "-no-log";
           "-no-links";
+        ]
+      else
+        [];
+
+      if OASISVersion.comparator_apply
+          (OASISVersion.version_of_string (ocaml_version ()))
+          (OASISVersion.VLesser (OASISVersion.version_of_string "3.11.1")) then
+        [
           "-install-lib-dir";
           (Filename.concat (standard_library ()) "ocamlbuild")
         ]
@@ -91,35 +97,32 @@ let fix_args args extra_argv =
 
 
 (** Run 'ocamlbuild -clean' if not already done *)
-let run_clean extra_argv =
+let run_clean ~ctxt extra_argv =
   let extra_cli =
     String.concat " " (Array.to_list extra_argv)
   in
-    (* Run if never called with these args *)
-    if not (BaseLog.exists ocamlbuild_clean_ev extra_cli) then
-      begin
-        OASISExec.run ~ctxt:!BaseContext.default
-          (ocamlbuild ()) (fix_args ["-clean"] extra_argv);
-        BaseLog.register ocamlbuild_clean_ev extra_cli;
-        at_exit
-          (fun () ->
-             try
-               BaseLog.unregister ocamlbuild_clean_ev extra_cli
-             with _ ->
-               ())
-      end
+  (* Run if never called with these args *)
+  if not (BaseLog.exists ~ctxt ocamlbuild_clean_ev extra_cli) then
+    begin
+      OASISExec.run ~ctxt (ocamlbuild ()) (fix_args ["-clean"] extra_argv);
+      BaseLog.register ~ctxt ocamlbuild_clean_ev extra_cli;
+      at_exit
+        (fun () ->
+           try
+             BaseLog.unregister ~ctxt ocamlbuild_clean_ev extra_cli
+           with _ -> ())
+    end
 
 
 (** Run ocamlbuild, unregister all clean events *)
-let run_ocamlbuild args extra_argv =
+let run_ocamlbuild ~ctxt args extra_argv =
   (* TODO: enforce that target in args must be UNIX encoded i.e. toto/index.html
-   *)
-  OASISExec.run ~ctxt:!BaseContext.default
-    (ocamlbuild ()) (fix_args args extra_argv);
+  *)
+  OASISExec.run ~ctxt (ocamlbuild ()) (fix_args args extra_argv);
   (* Remove any clean event, we must run it again *)
   List.iter
-    (fun (e, d) -> BaseLog.unregister e d)
-    (BaseLog.filter [ocamlbuild_clean_ev])
+    (fun (e, d) -> BaseLog.unregister ~ctxt e d)
+    (BaseLog.filter ~ctxt [ocamlbuild_clean_ev])
 
 
 (** Determine real build directory *)
@@ -127,13 +130,13 @@ let build_dir extra_argv =
   let rec search_args dir =
     function
       | "-build-dir" :: dir :: tl ->
-          search_args dir tl
+        search_args dir tl
       | _ :: tl ->
-          search_args dir tl
+        search_args dir tl
       | [] ->
-          dir
+        dir
   in
-    search_args "_build" (fix_args [] extra_argv)
+  search_args "_build" (fix_args [] extra_argv)
 
 
 (* END EXPORT *)
@@ -141,7 +144,7 @@ let build_dir extra_argv =
 open OASISValues
 
 let fix_build_tools tool pkg =
-  let fix_build_tools' sct bs =
+  let fix_build_tools' _ bs =
     if not (List.mem tool bs.bs_build_tools) then
       {bs with bs_build_tools = tool :: bs.bs_build_tools}
     else
@@ -154,44 +157,39 @@ let fix_build_tools tool pkg =
          let sct =
            match sct with
              | Executable (cs, bs, exec) ->
-                 let bs = fix_build_tools' sct bs in
-                   Executable (cs, bs, exec)
+               let bs = fix_build_tools' sct bs in
+               Executable (cs, bs, exec)
 
              | Library (cs, bs, lib) ->
-                 let bs = fix_build_tools' sct bs in
-                   Library (cs, bs, lib)
+               let bs = fix_build_tools' sct bs in
+               Library (cs, bs, lib)
 
              | Object (cs, bs, obj) ->
-                 let bs = fix_build_tools' sct bs in
-                   Object (cs, bs, obj)
+               let bs = fix_build_tools' sct bs in
+               Object (cs, bs, obj)
 
              | Flag _ | SrcRepo _ | Test _ | Doc _ as sct ->
-                 sct
+               sct
          in
-           sct :: acc)
+         sct :: acc)
       []
       pkg.sections
   in
-    {pkg with sections = List.rev sections}
+  {pkg with sections = List.rev sections}
 
 
 module Tag =
 struct
   (** [filename_concat fn1 fn2] Concat filename, using semantic of _tags
       [fn1] must be a real filename whereas fn2 can contains wildcards.
-    *)
+  *)
   let filename_concat fn1 fn2 =
     OASISUnixPath.concat (OASISUnixPath.reduce fn1) fn2
 end
 
 (** Check OCaml version constraint defined in _oasis. *)
 let check_ocaml_version version pkg =
-  match pkg.ocaml_version with
-    | Some ocaml_version ->
-        let min_ocaml_version = OASISVersion.version_of_string version in
-        OASISVersion.comparator_ge min_ocaml_version ocaml_version
-    | None ->
-        false
+  OASISVersion.StringVersion.comparator_ge version pkg.ocaml_version
 
 
 let ocamlbuild_more_args =
@@ -209,7 +207,7 @@ type ocamlbuild_common =
   {
     plugin_tags: string option;
     extra_args: string list;
-  } with odn
+  }
 
 
 let ocamlbuild_common_generator pivot_data schm id =
@@ -233,26 +231,30 @@ let ocamlbuild_common_generator pivot_data schm id =
       (fun () -> s_ "Gives extra arguments to ocamlbuild")
       pivot_data (fun _ t -> t.extra_args)
   in
-    fun data ->
-      {
-        extra_args = extra_args data;
-        plugin_tags = plugin_tags data;
-      }
+  fun data ->
+    {
+      extra_args = extra_args data;
+      plugin_tags = plugin_tags data;
+    }
 
 
-let extra_args_ocamlbuild_common ~ctxt pkg t =
-  let extra_args =
-    if t.plugin_tags <> None &&
-       not (ocamlbuild_supports_plugin_tags pkg) then begin
-      OASISMessage.error
-        ~ctxt:ctxt
-        (f_ "'XOCamlbuildPluginTags' in only available for OCaml >= 4.01. \
-             Please restrict your requirements with 'OCamlVersion: >= 4.01'");
-      t.extra_args
+let extra_args_ocamlbuild_common ctxt pkg t =
+  let supports_plugins = ocamlbuild_supports_plugin_tags pkg in
+  let extra_args, ctxt =
+    if t.plugin_tags <> None && (not supports_plugins) then begin
+      t.extra_args,
+      OASISPlugin.set_error
+        (not supports_plugins)
+        (s_ "'XOCamlbuildPluginTags' in only available for OCaml >= 4.01. \
+             Please restrict your requirements with 'OCamlVersion: >= 4.01'")
+        ctxt
     end else begin
-      match t.plugin_tags with
+      let extra_args =
+        match t.plugin_tags with
         | Some tags -> "-plugin-tags" :: ("'" ^ tags ^ "'") :: t.extra_args
         | None -> t.extra_args
+      in
+      extra_args, ctxt
     end
   in
   let extra_args =
@@ -261,4 +263,4 @@ let extra_args_ocamlbuild_common ~ctxt pkg t =
     else
       extra_args
   in
-  extra_args
+  extra_args, ctxt
